/*
 * Copyright (c) 2015 Jonas Danielsson
 *
 * GNOME Radio is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * GNOME Radio is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with GNOME Radio; if not, see <http://www.gnu.org/licenses/>.
 *
 * Author: Jonas Danielsson <jonas@threetimestwo.org>
 */

#ifndef __RADIO_CONTACT_STORE_H__
#define __RADIO_CONTACT_STORE_H__

#include <glib-object.h>
#include "radio-contact.h"

#define RADIO_TYPE_CONTACT_STORE radio_contact_store_get_type ()
G_DECLARE_FINAL_TYPE(RadioContactStore, radio_contact_store, RADIO, CONTACT_STORE,
                     GObject)

typedef struct _RadioContactStorePrivate RadioContactStorePrivate;

/**
 * RadioContactStoreState:
 * @RADIO_CONTACT_STORE_STATE_INITIAL: Initial state
 * @RADIO_CONTACT_STORE_STATE_LOADING: Loading
 * @RADIO_CONTACT_STORE_STATE_LOADED: Loaded
 */
typedef enum {
  RADIO_CONTACT_STORE_STATE_INITIAL,
  RADIO_CONTACT_STORE_STATE_LOADING,
  RADIO_CONTACT_STORE_STATE_LOADED,
} RadioContactStoreState;

/**
 * RadioContactStoreLookupCallback:
 * @contact: A #RadioContact object
 */
typedef void (*RadioContactStoreLookupCallback) (RadioContact *contact);

struct _RadioContactStore
{
  GObject parent_instance;
  RadioContactStorePrivate *priv;
};

struct _RadioContactStoreClass
{
  GObjectClass parent_class;
};

RadioContactStore * radio_contact_store_new          (void);
void               radio_contact_store_load         (RadioContactStore              *store);
void               radio_contact_store_lookup       (RadioContactStore              *store,
                                                    const char                    *id,
                                                    RadioContactStoreLookupCallback callback);
GList *            radio_contact_store_get_contacts (RadioContactStore *store);

#endif
