/*
 * Copyright (c) 2015 Marcus Lundblad
 *
 * GNOME Radio is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * GNOME Radio is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with GNOME Radio; if not, see <http://www.gnu.org/licenses/>
 *
 * Author: Marcus Lundblad <ml@update.uu.se>
 */

#include "radio-osm-way.h"

struct _RadioOSMWayPrivate
{
  GArray *node_ids;
};

G_DEFINE_TYPE_WITH_PRIVATE (RadioOSMWay, radio_osm_way, RADIO_TYPE_OSMOBJECT);

static void
radio_osm_way_dispose (GObject *object)
{
  RadioOSMWay *way = RADIO_OSMWAY (object);

  g_array_free (way->priv->node_ids, TRUE);
  way->priv->node_ids = NULL;

  G_OBJECT_CLASS (radio_osm_way_parent_class)->dispose (object);
}

static const char *
radio_osm_way_get_xml_tag_name (void)
{
  return "way";
}

static xmlNodePtr
radio_osm_way_create_node_xml_node (guint64 ref)
{
  xmlNodePtr nd;
  char buf[16];

  g_snprintf (buf, 16, "%" G_GUINT64_FORMAT, ref);
  nd = xmlNewNode (NULL, (xmlChar *) "nd");
  xmlNewProp (nd, (xmlChar *) "ref", (xmlChar *) buf);

  return nd;
}

static xmlNodePtr
radio_osm_way_get_xml_child_nodes(const RadioOSMObject *object)
{
  const RadioOSMWay *way = RADIO_OSMWAY ((RadioOSMObject *) object);
  int i;
  xmlNodePtr result;
  xmlNodePtr next;

  g_return_val_if_fail (way->priv->node_ids->len > 0, NULL);

  result = radio_osm_way_create_node_xml_node (g_array_index (way->priv->node_ids,
                                                             guint64, 0));
  next = result;
  
  for (i = 1; i < way->priv->node_ids->len; i++)
    {
      xmlNodePtr new_node;
      new_node =
        radio_osm_way_create_node_xml_node (g_array_index (way->priv->node_ids,
                                                          guint64, i));
      next = xmlAddNextSibling (next, new_node);
    }

  return result;
}

static void
radio_osm_way_class_init (RadioOSMWayClass *klass)
{
  GObjectClass *way_class = G_OBJECT_CLASS (klass);
  RadioOSMObjectClass *object_class = RADIO_OSMOBJECT_CLASS (klass);
  
  way_class->dispose = radio_osm_way_dispose;
  object_class->get_xml_tag_name = radio_osm_way_get_xml_tag_name;
  object_class->get_xml_child_nodes = radio_osm_way_get_xml_child_nodes;
}

static void
radio_osm_way_init (RadioOSMWay *way)
{
  way->priv = radio_osm_way_get_instance_private (way);
  way->priv->node_ids = g_array_new (FALSE, FALSE, sizeof (guint64));
}

RadioOSMWay *
radio_osm_way_new (guint64 id, guint version, guint64 changeset)
{
  return g_object_new (RADIO_TYPE_OSMWAY,
                       "id", id,
                       "version", version,
                       "changeset", changeset, NULL);
}

void
radio_osm_way_add_node_id (RadioOSMWay *way, guint64 id)
{
  g_array_append_val (way->priv->node_ids, id);
}
